/* $Id: sound.cpp,v 1.3 2006/09/13 15:09:46 keko Exp $ */
/*
   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.

   See the COPYING file for more details.
*/


#include "sound.h"

Sound::Sound()
{
}

Sound::~Sound()
{
	close_sound();
}

bool Sound::init_sound() {
//sounds don't sound good on Windows unless the buffer size is 4k,
//but this seems to cause crashes on other systems...
#ifdef WIN32
	const size_t buf_size = 4096;
#else
	const size_t buf_size = 1024;
#endif
	if(SDL_WasInit(SDL_INIT_AUDIO) == 0)
		if(SDL_InitSubSystem(SDL_INIT_AUDIO) == -1)
			return false;

	if(!mix_ok) {
		if(Mix_OpenAudio(MIX_DEFAULT_FREQUENCY,MIX_DEFAULT_FORMAT,2,buf_size) == -1) {
			mix_ok = false;
			printf("Could not initialize audio: %s\n",Mix_GetError());
			return false;
		}

		mix_ok = true;
		Mix_AllocateChannels(16);
 		set_sound_volume(999);
 		set_music_volume(999);

		printf("Audio initialized.\n");

		play_music("share/glparchis/ambiental.ogg");
	}
	return true;
}

void Sound::close_sound() {
	int numtimesopened, frequency, channels;
	Uint16 format;
	if(mix_ok) {
		stop_sound();
		stop_music();
		mix_ok = false;

		numtimesopened = Mix_QuerySpec(&frequency, &format, &channels);
		if(numtimesopened == 0) {
			printf( "Error closing audio device: %s", Mix_GetError() );
		}
		while (numtimesopened) {
			Mix_CloseAudio();
			--numtimesopened;
		}
	}
	if(SDL_WasInit(SDL_INIT_AUDIO) != 0)
		SDL_QuitSubSystem(SDL_INIT_AUDIO);

	printf("Audio device released.\n");
}

void Sound::stop_music() {
	if(mix_ok) {
		Mix_HaltMusic();

		std::map<std::string,Mix_Music*>::iterator i;
		for(i = music_cache.begin(); i != music_cache.end(); ++i)
			Mix_FreeMusic(i->second);
		music_cache.clear();
	}
}

void Sound::stop_sound() {
	if(mix_ok) {
		Mix_HaltChannel(-1);

		std::map<std::string,Mix_Chunk*>::iterator i;
		for(i = sound_cache.begin(); i != sound_cache.end(); ++i)
			Mix_FreeChunk(i->second);
		sound_cache.clear();
	}
}

void Sound::play_music(char * file)
{
Mix_PlayingMusic();
/*	if( && current_music == file)
		return;
	else if(current_music=="" && file=="")
		return;

	if(file=="")
		file = current_music;
	else
		current_music = file;*/

	if( mix_ok) {
		std::map<std::string,Mix_Music*>::const_iterator itor = music_cache.find(file);
		if(itor == music_cache.end()) {
          char configprefix[1000];
          sprintf(configprefix,"%s",PREFIX);
          if (strcmp(configprefix,"NONE")==0){
            sprintf(configprefix,"/usr/local");
          }
          char filename[1000];
          sprintf (filename,"%s/%s",configprefix,file);
// 			const std::string& filename = NULL;//get_binary_file_location("music",file);

			if(filename==NULL) {
				return;
			}

			Mix_Music* const music = Mix_LoadMUS(filename);
			if(music == NULL) {
				printf("Could not load music file '%s': %s\n",(const char *)filename,	Mix_GetError());
				return;
			}

			itor = music_cache.insert(std::pair<std::string,Mix_Music*>(file,music)).first;
		}

		if(Mix_PlayingMusic()) {
			Mix_FadeOutMusic(500);
		}

		const int res = Mix_FadeInMusic(itor->second,-1,500);
		if(res < 0) {
			printf( "Could not play music: %s %s\n", Mix_GetError(),file);
		}
	}
}

void Sound::play_sound(char * file)
{
//    printf("Haciendo sonar el fichero %s\n",file);
	if( mix_ok) {
		// the insertion will fail if there is already an element in the cache
		std::pair< std::map< std::string, Mix_Chunk * >::iterator, bool >
			it = sound_cache.insert(std::make_pair(file, (Mix_Chunk *)0));
		Mix_Chunk *&cache = it.first->second;
		if (it.second) {
           char configprefix[1000];
  sprintf(configprefix,"%s",PREFIX);
  if (strcmp(configprefix,"NONE")==0){
    sprintf(configprefix,"/usr/local");
  }
  char filename[1000];
  sprintf (filename,"%s/%s",configprefix,file);
// 			std::string const &filename = NULL;//get_binary_file_location("sounds", file);
			if (filename) {
#ifdef USE_ZIPIOS
				std::string const &s = read_file(filename);
				if (!s.empty()) {
					SDL_RWops* ops = SDL_RWFromMem((void*)s.c_str(), s.size());
					cache = Mix_LoadWAV_RW(ops,0);
				}
#else
				cache = Mix_LoadWAV(filename);
#endif
			}

			if (cache == NULL) {
				printf( "Could not load sound file '%s': %s",filename, Mix_GetError());
				return;
			}
		}

		//play on the first available channel
		const int res = Mix_PlayChannel(-1, cache, 0);
		if(res < 0) {
			printf("error playing sound effect: %s\n",Mix_GetError());
		}
	}
}

void Sound::set_music_volume(int vol)
{
	if(mix_ok && vol >= 0) {
		if(vol > MIX_MAX_VOLUME)
			vol = MIX_MAX_VOLUME;
		Mix_VolumeMusic(vol);
	}
}

void Sound::set_sound_volume(int vol)
{
	if(mix_ok && vol >= 0) {
		if(vol > MIX_MAX_VOLUME)
			vol = MIX_MAX_VOLUME;
		Mix_Volume(-1, vol);
	}
}

